/*
 * Copyright (C) 2007-2011 Linutop SAS <http://www.linutop.com/>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <glib/gprintf.h>
#include <stdlib.h>

#include "libappstarter/as.h"

#define PLUGIN "slideshow"

enum {
    FS = 0,
    RND,
};

typedef struct _SlPlugin {
    int fullscreen;
    int random;
    int interval;
    gchar *path;
} SlPlugin;

SlPlugin *plugin;
gchar *cmd = NULL;
gchar *conffile = NULL;

static void
read_config ()
{
    XfceRc *rc;
    rc = as_rc_open (conffile, TRUE);
    if (!rc)
        return;
    xfce_rc_set_group (rc, PLUGIN);
    plugin->random = xfce_rc_read_int_entry (rc, "random", 0);
    plugin->fullscreen = xfce_rc_read_int_entry (rc, "fullscreen", 1);
    plugin->interval = xfce_rc_read_int_entry (rc, "interval", 15);
    plugin->path = g_strdup (xfce_rc_read_entry (
        rc, "path", g_getenv ("HOME")));
    xfce_rc_close (rc);
}

static void
write_config ()
{
    XfceRc *rc;
    rc = as_rc_open (conffile, FALSE);
    if (!rc)
        return;
    xfce_rc_set_group (rc, PLUGIN);
    xfce_rc_write_int_entry (rc, "random", plugin->random);
    xfce_rc_write_int_entry (rc, "fullscreen", plugin->fullscreen);
    xfce_rc_write_int_entry (rc, "interval", plugin->interval);
    xfce_rc_write_entry (rc, "path", plugin->path);
    xfce_rc_close (rc);
}

/* callbacks */
static void
entry_changed (GtkWidget *fe, gpointer data)
{
    g_free (plugin->path);
    plugin->path = g_strdup (
        as_file_entry_get_filename (AS_FILE_ENTRY (fe)));
    write_config();
}

static void
spin_changed (GtkSpinButton *spin, gpointer data)
{
    plugin->interval = (int) gtk_spin_button_get_value (spin);
    write_config();
}

static void
chk_toggled (GtkWidget *btn, gpointer data)
{
    int id;
    id = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (btn), "as-id"));

    switch (id) {
        case FS:
            plugin->fullscreen = (gtk_toggle_button_get_active (
                GTK_TOGGLE_BUTTON (btn))) ? 1 : 0;
            break;
        case RND:
            plugin->random = (gtk_toggle_button_get_active (
                GTK_TOGGLE_BUTTON (btn))) ? 1 : 0;
            break;
    }
    write_config ();
}

GtkWidget *
get_config_widget (gchar *file)
{
    GtkWidget *vbox, *hbox;
    GtkWidget *fs_chk, *rnd_chk, *int_spin;
    GtkWidget *lbl;
    GtkObject *adj;
    GtkWidget *fe;

    conffile = g_strdup (file);
    plugin = g_new0(SlPlugin, 1);
    plugin->path = NULL;
    read_config();

    vbox = gtk_vbox_new (FALSE, 2);
    hbox = gtk_hbox_new (FALSE, 2);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 2);
    gtk_widget_show (hbox);
    lbl = gtk_label_new (_("Select a folder: "));
    gtk_box_pack_start (GTK_BOX (hbox), lbl, FALSE, FALSE, 2);
    gtk_widget_show (lbl);
    fe = as_file_entry_new (_("Select a folder"),
                            GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
                            plugin->path);
    gtk_box_pack_start (GTK_BOX (hbox), fe, TRUE, TRUE, 2);

    hbox = gtk_hbox_new (FALSE, 2);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 2);
    gtk_widget_show (hbox);
    lbl = gtk_label_new (_("Interval (in seconds): "));
    gtk_box_pack_start (GTK_BOX (hbox), lbl, FALSE, FALSE, 2);
    gtk_widget_show (lbl);
    adj = gtk_adjustment_new ((gdouble) plugin->interval, 1, 3600,
                              1, 1, 1);
    int_spin = gtk_spin_button_new (GTK_ADJUSTMENT (adj), 1, 0);
    gtk_box_pack_start (GTK_BOX (hbox), int_spin, FALSE, FALSE, 2);
    gtk_spin_button_set_value (GTK_SPIN_BUTTON (int_spin),
        plugin->interval);
    gtk_widget_show (int_spin);

    fs_chk = gtk_check_button_new_with_label (_("Play in full screen"));
    g_object_set_data (G_OBJECT (fs_chk), "as-id", GUINT_TO_POINTER (FS));
    gtk_box_pack_start (GTK_BOX (vbox), fs_chk, FALSE, FALSE, 2);
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (fs_chk),
        (plugin->fullscreen) ? TRUE : FALSE);
    gtk_widget_show (fs_chk);
    rnd_chk = gtk_check_button_new_with_label (_("Random order"));
    g_object_set_data (G_OBJECT (rnd_chk), "as-id", GUINT_TO_POINTER (RND));
    gtk_box_pack_start (GTK_BOX (vbox), rnd_chk, FALSE, FALSE, 2);
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rnd_chk),
        (plugin->random) ? TRUE : FALSE);
    gtk_widget_show (rnd_chk);

    g_signal_connect (G_OBJECT (fe), "changed",
        G_CALLBACK (entry_changed), NULL);
    g_signal_connect (G_OBJECT (int_spin), "value-changed",
        G_CALLBACK (spin_changed), NULL);
    g_signal_connect (G_OBJECT (fs_chk), "toggled",
        G_CALLBACK (chk_toggled), NULL);
    g_signal_connect (G_OBJECT (rnd_chk), "toggled",
        G_CALLBACK (chk_toggled), NULL);

    return vbox;
}

gchar *
get_cmd()
{
    g_free (cmd);
    cmd = g_strconcat (
               "xslideshow -override -perfect -animate puzzle -interval ",
               plugin->interval, " ",
               (plugin->fullscreen) ? "-full " : "",
               (plugin->random) ? "-selection random " : "",
               plugin->path, "/*",
               NULL);
    return cmd;
}

void
free_all()
{
    g_free (cmd);
    g_free (plugin->path);
    g_free (plugin);
    g_free (conffile);
}
